--
-- NewCurrencies
--
-- @author Rockstar
-- @date 10/02/2021
--


NewCurrencies = {
	MOD_DIRECTORY = g_currentModDirectory,
	MOD_NAME = g_currentModName,
	CONFIG_FILENAME = g_currentModDirectory .. "xml/currencies.xml",
	SETTINGS_DIRECTORY = getUserProfileAppPath() .. "modsSettings",
	SETTINGS_FILENAME = getUserProfileAppPath() .. "modsSettings/currency.xml"
}

function NewCurrencies:loadMap(filename)
	local gameInfoDisplay = g_currentMission.hud.gameInfoDisplay

	self.defaultMaxDisplayValue = I18N.MONEY_MAX_DISPLAY_VALUE
	self.defaultMoneyBoxWidth = gameInfoDisplay.moneyBox:getWidth()
	self.defaultMoneyTextWidth = getTextWidth(gameInfoDisplay.moneyTextSize, g_i18n:formatNumber(self.defaultMaxDisplayValue))

	local currencies, texts, useConverter = self:loadCurrenciesFromXMLFile(NewCurrencies.CONFIG_FILENAME, g_settingsScreen.settingsModel:getMoneyUnitTexts())

	self.currencies = currencies
	self.useConverter = useConverter

	local pageSettingsGeneral = g_currentMission.inGameMenu.pageSettingsGeneral
	local moneyUnitElement = pageSettingsGeneral.multiMoneyUnit
	local global_i18n = getfenv(0).g_i18n

	pageSettingsGeneral.optionMapping[moneyUnitElement] = nil
	pageSettingsGeneral.onFrameClose = Utils.appendedFunction(pageSettingsGeneral.onFrameClose, self.saveToXMLFile)
	FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, self.saveToXMLFile)
	moneyUnitElement.onClickCallback = Utils.overwrittenFunction(moneyUnitElement.onClickCallback, self.onClickCallback)
	gameInfoDisplay.setMoneyUnit = Utils.overwrittenFunction(gameInfoDisplay.setMoneyUnit, self.setMoneyUnit)
	global_i18n.getCurrencySymbol = Utils.overwrittenFunction(global_i18n.getCurrencySymbol, self.getCurrencySymbol)
	global_i18n.formatMoney = Utils.overwrittenFunction(global_i18n.formatMoney, self.formatMoney)

	createFolder(NewCurrencies.SETTINGS_DIRECTORY)

	local state = self:loadStateFromXMLFile()

	g_currentMission:setMoneyUnit(state)
	moneyUnitElement:setState(state)
	moneyUnitElement:setTexts(texts)

	local shopMenu = g_currentMission.shopMenu
	local offsetX, offsetY = getNormalizedScreenValues(-15, 0)

	shopMenu.pageShopItemDetails.currentBalanceLabel:move(offsetX, offsetY)
	shopMenu.pageShopGarageOwned.currentBalanceLabel:move(offsetX, offsetY)
	shopMenu.pageShopGarageLeased.currentBalanceLabel:move(offsetX, offsetY)

	addConsoleCommand("gsEnableCurrencyConverter", "", "consoleCommandEnableCurrencyConverter", self)
end

function NewCurrencies:deleteMap()
	removeConsoleCommand("gsEnableCurrencyConverter")
end

function NewCurrencies:saveStateToXMLFile()
	local currencyXMLFile = createXMLFile("currencyXMLFile", NewCurrencies.SETTINGS_FILENAME, "currency")

	if currencyXMLFile ~= nil then
		setXMLInt(currencyXMLFile, "currency.currency", g_i18n.moneyUnit)
		setXMLBool(currencyXMLFile, "currency.useConverter", self.useConverter)

		saveXMLFile(currencyXMLFile)
		delete(currencyXMLFile)
	end
end

function NewCurrencies:loadStateFromXMLFile()
	local state = g_gameSettings:getValue(GameSettings.SETTING.MONEY_UNIT)
	local xmlFilename = NewCurrencies.SETTINGS_FILENAME

	if fileExists(xmlFilename) then
		local currencyXMLFile = loadXMLFile("currencyXMLFile", xmlFilename)

		if currencyXMLFile ~= nil then
			local currency = Utils.getNoNil(getXMLInt(currencyXMLFile, "currency.currency"), state)

			if currency <= #self.currencies then
				state = currency
			end

			self.useConverter = Utils.getNoNil(getXMLBool(currencyXMLFile, "currency.useConverter"), self.useConverter)
			delete(currencyXMLFile)
		end
	end

	return state
end

function NewCurrencies:loadCurrenciesFromXMLFile(xmlFilename, defaultTexts)
	local currencies = {}
	local texts = defaultTexts
	local useConverter = false
	local xmlFile = loadXMLFile("currenciesXMLFile", xmlFilename)

	if xmlFile ~= nil then
		local imageFilename = getXMLString(xmlFile, "currencies#imageFilename")

		if imageFilename ~= nil then
			self.imageFilename = NewCurrencies.MOD_DIRECTORY .. imageFilename
		end
 
		useConverter = Utils.getNoNil(getXMLBool(xmlFile, "currencies#useConverter"), useConverter)

		currencies[1] = {prefix = false, factor = Utils.getNoNil(getXMLFloat(xmlFile, "currencies#euroFactor"), 1), maxDisplayValue = self.defaultMaxDisplayValue, isDefaultCurrency = true}
		currencies[2] = {prefix = true, factor = Utils.getNoNil(getXMLFloat(xmlFile, "currencies#dolarFactor"), 1.34), maxDisplayValue = self.defaultMaxDisplayValue, isDefaultCurrency = true}
		currencies[3] = {prefix = true, factor = Utils.getNoNil(getXMLFloat(xmlFile, "currencies#poundFactor"), 0.79), maxDisplayValue = self.defaultMaxDisplayValue, isDefaultCurrency = true}

		local i = 0

		while true do
			local key = string.format("currencies.currency(%d)", i)

			if not hasXMLProperty(xmlFile, key) then
				break
			end

			local unit = XMLUtil.getXMLI18NValue(xmlFile, key .. "#text", getXMLString, nil, "", NewCurrencies.MOD_NAME, true)
			local unitShort = XMLUtil.getXMLI18NValue(xmlFile, key .. "#textShort", getXMLString, nil, "", NewCurrencies.MOD_NAME, true)
			local prefix = Utils.getNoNil(getXMLBool(xmlFile, key .. "#prefixSymbol"), true)
			local factor = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#factor"), 1)
			local a, b, c, d = StringUtil.getVectorFromString(Utils.getNoNil(getXMLString(xmlFile, key .. "#imageUVs"), "0 0 0 0"))
			local maxDisplayValue = Utils.getNoNil(getXMLString(xmlFile, key .. "#maxDisplayValue"), self.defaultMaxDisplayValue)

			table.insert(texts, unit)
			table.insert(currencies, {unit = unit, unitShort = unitShort, prefix = prefix, factor = factor, uvs = {a, b, c, d}, maxDisplayValue = tonumber(maxDisplayValue), isDefaultCurrency = false})

			i = i + 1
		end

		delete(xmlFile)
	end

	return currencies, texts, useConverter
end

function NewCurrencies:updateMoneyBoxWidth(currency)
	local maxDisplayValue = self.defaultMaxDisplayValue

	if self.useConverter then
		maxDisplayValue = currency.maxDisplayValue
	end

	if maxDisplayValue ~= nil and maxDisplayValue ~= I18N.MONEY_MAX_DISPLAY_VALUE then
		I18N.MONEY_MAX_DISPLAY_VALUE, I18N.MONEY_MIN_DISPLAY_VALUE = maxDisplayValue, -maxDisplayValue

		local gameInfoDisplay = g_currentMission.hud.gameInfoDisplay
		local maxMoneyTextWidth = getTextWidth(gameInfoDisplay.moneyTextSize, g_i18n:formatNumber(maxDisplayValue))
		local extraWidth = maxMoneyTextWidth - self.defaultMoneyTextWidth

		gameInfoDisplay.moneyBox.overlay.width = self.defaultMoneyBoxWidth + extraWidth
		gameInfoDisplay:storeScaledValues()
		gameInfoDisplay:updateSizeAndPositions()
	end
end

function NewCurrencies:updateMoneyTexts()
	for _, name in pairs({"button_borrow5000", "button_repay5000", "helpText_57_part01", "hint_17"}) do
		getfenv(0).g_i18n:setText(name, string.format(g_i18n:getText("newCurrencies_global_" .. name), g_i18n:formatMoney(5000)))
	end
end

function NewCurrencies:consoleCommandEnableCurrencyConverter()
	self.useConverter = not self.useConverter

	self:saveStateToXMLFile()

	self:updateMoneyTexts()
	self:updateMoneyBoxWidth(self.currencies[g_i18n.moneyUnit])

	return "CurrencyConverterEnabled = " .. tostring(self.useConverter)
end

function NewCurrencies.saveToXMLFile(object)
	NewCurrencies:saveStateToXMLFile()
end

function NewCurrencies.onClickCallback(object, superFunc, state, checkboxElement)
	g_currentMission:setMoneyUnit(state)
end

function NewCurrencies.setMoneyUnit(object, superFunc, moneyUnit)
	local currency = NewCurrencies.currencies[moneyUnit]
	local pixelUVs, refSize = GameInfoDisplay.UV.MONEY_ICON[moneyUnit], {1024, 1024}
	local overlayFilename = g_baseHUDFilename

	if not currency.isDefaultCurrency then
		overlayFilename = Utils.getNoNil(NewCurrencies.imageFilename, overlayFilename)
		pixelUVs, refSize = currency.uvs, {256, 256}
	end

	object.moneyIconOverlay:setImage(overlayFilename)
	object.moneyIconOverlay:setUVs(getNormalizedUVs(pixelUVs, refSize))
	object.moneyUnit = moneyUnit

	NewCurrencies:updateMoneyTexts()
	NewCurrencies:updateMoneyBoxWidth(currency)
end

function NewCurrencies.getCurrencySymbol(object, superFunc, useShort)
	local currency = NewCurrencies.currencies[object.moneyUnit]

	if currency.isDefaultCurrency then
		return superFunc(object, useShort)
	else
		local text = currency.unit

		if useShort then
			text = currency.unitShort
		end

		return text
	end
end

function NewCurrencies.formatMoney(object, superFunc, number, precision, addCurrency, prefixCurrencySymbol)
	local currency = NewCurrencies.currencies[object.moneyUnit]

	if addCurrency == nil or addCurrency then
		prefixCurrencySymbol = Utils.getNoNil(currency.prefix, true)
	end

	if NewCurrencies.useConverter then
		number = number * currency.factor
	end

	return superFunc(object, number, precision, addCurrency, prefixCurrencySymbol)
end

addModEventListener(NewCurrencies)